@@ -37,7 +37,7 @@ void printlistfunc(FILE *fout)
   for(i=0;lfunc[i].gpname;i++)
     fprintf(fout,"%s %s %s\n",lfunc[i].gpname,lfunc[i].proto.cname,(lfunc[i].proto.code?lfunc[i].proto.code:""));
 }
-void checkisvar(int aff, char *func, int *var, Gtype *t)
+void checkisvar(int aff, const char *func, int *var, Gtype *t)
 {
   *var=aff;
   switch(tree[*var].f)
@@ -56,7 +56,7 @@ void checkisvar(int aff, char *func, int *var, Gtype *t)
   }
 }
 
-void genequal(int aff, char *func, int *var, int *binf, Gtype *t)
+void genequal(int aff, const char *func, int *var, int *binf, Gtype *t)
 {
   if (tree[aff].f!=Faffect)
     die(aff,"incorrect syntax for %s",func);
@@ -131,9 +131,8 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
   int nb;
   int y=tree[n].y;
   int binf,bsup,seq;
-  int var,vinf,vsup;
+  int var,vinf,vsup,vprime;
   int bseq=-1,aseq=-1,ret=-1;
-  int val,vprime;
   Gtype tv;
   int gpm,bl1,bl2;
   int savx;
@@ -158,19 +157,18 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
     {
       /*if(1,...) expr are often used to create block. We honor it here*/
       if (p>=0 && tree[p].f!=Fseq)
-	newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&ret);
+	newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&ret);
       arg[1]=geninsertvar(arg[1],ret);
       genblock(arg[1],-1);
       aseq=arg[1];
     }
     else
     {
-      if(nb==3)
-	val=-1;
-      else
-	val=GNIL;
       if (p>=0 && tree[p].f!=Fseq)
-	newdecl((1<<Cauto)|(1<<Cimmutable),Gvoid,val,&ret);
+        if(nb==3)
+	  newdecl((1<<Cauto)|(1<<Cimmutable),Gempty,-1,&ret);
+        else
+	  newdecl((1<<Cauto)|(1<<Cimmutable),Gvoid,GNIL,&ret);
       genblock(arg[0],n);
       arg[1]=geninsertvar(arg[1],ret);
       genblock(arg[1],-1);
@@ -186,30 +184,37 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
   case GPinstall:
     {
       gpfunc *gp;
-      char *p;
+      const char *p;
       int d, nf;
+      /* We should not use genblock(y,n) since prototype is
+       * {rrD\"\",r,D\"\",s,}
+       * Instead we fix Fentry to Fconst.
+       */
       genblock(y,n);
       nb=genlistargs(n,arg,2,4);
       /*What to do with arg[3]? add a linking ??*/
       if (nb<3 || arg[2]==GNOARG) arg[2]=arg[0];
       d=detag(arg[2]);
-      if (tree[d].f!=Fentry && tree[d].f!=Fstring)
+      if (tree[d].f!=Fentry && !is_const(d,CSTstr))
         die(n,"Incorrect GP name in install");
+      tree[d].f=Fconst;
       nf=getfunc(entryname(d)); gp=lfunc+nf;
       d=detag(arg[0]);
-      if (tree[d].f!=Fentry && tree[d].f!=Fstring)
+      if (tree[d].f!=Fentry && !is_const(d,CSTstr))
         die(n,"Incorrect C name in install");
+      tree[d].f=Fconst;
       gp->proto.cname=entryname(d);
       if (nb==4 && arg[3]!=GNOARG)
       {
 	d=detag(arg[3]);
-	if (tree[d].f!=Fentry && tree[d].f!=Fstring)
+	if (!is_const(d,CSTstr))
 	  die(n,"Incorrect library name in install");
 	gp->proto.origin=entryname(d);
       }
       d=detag(arg[1]);
-      if (tree[d].f!=Fentry && tree[d].f!=Fstring)
+      if (tree[d].f!=Fentry && !is_const(d,CSTstr))
         die(n,"Incorrect code in install");
+      tree[d].f=Fconst;
       p=entryname(d);
       gp->proto.code=p;
       if (gp->proto.code[0]=='v')
@@ -237,9 +242,9 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
     nb=genlistargs(n,arg,3,3);
     genequal(arg[0],gp->gpname,&var,&binf,&tv);
     bsup=arg[1];seq=arg[2];
-    newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&vinf);
+    newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&vinf);
     bseq=addseqright(bseq,geninsertvar(binf,newleaf(vinf)));
-    newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&vsup);
+    newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&vsup);
     bseq=addseqright(bseq,geninsertvar(bsup,newleaf(vsup)));
     genblock(bseq,-1);
     savx=s_ctx.n; 
@@ -309,11 +314,11 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
       nb=genlistargs(n,arg,4,4);
       genequal(arg[0],gp->gpname,&var,&binf,&tv);
       bsup=arg[1];bstep=arg[2];seq=arg[3];
-      newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&vinf);
+      newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&vinf);
       bseq=addseqright(bseq,geninsertvar(binf,newleaf(vinf)));
-      newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&vsup);
+      newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&vsup);
       bseq=addseqright(bseq,geninsertvar(bsup,newleaf(vsup)));
-      newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&vstep);
+      newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&vstep);
       bseq=addseqright(bseq,geninsertvar(bstep,newleaf(vstep)));
       if (tree[bstep].f==Fmat || (tree[bstep].f==Ftag && tree[bstep].y==Gvec))
       {
@@ -391,9 +396,9 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
     nb=genlistargs(n,arg,3,3);
     genequal(arg[0],gp->gpname,&var,&binf,&tv);
     bsup=arg[1];seq=arg[2];
-    newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&vinf);
+    newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&vinf);
     bseq=addseqright(bseq,geninsertvar(binf,newleaf(vinf)));
-    newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&vsup);
+    newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&vsup);
     bseq=addseqright(bseq,geninsertvar(bsup,newleaf(vsup)));
     genblock(bseq,-1);
     savx=s_ctx.n; 
@@ -435,9 +440,9 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
     bsup=arg[1];seq=arg[2];
     if(nb==3)
       arg[3]=newnode(Fsmall,(gp->spec==GPsum?0:1),-1);
-    newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&vinf);
+    newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&vinf);
     bseq=addseqright(bseq,geninsertvar(binf,newleaf(vinf)));
-    newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&vsup);
+    newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&vsup);
     bseq=addseqright(bseq,geninsertvar(bsup,newleaf(vsup)));
     newdecl(0,Ggen,-1,&ret);
     bseq=addseqright(bseq,geninsertvar(arg[3],newleaf(ret)));
@@ -527,7 +532,7 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
   case GPvectorsmall:
     nb=genlistargs(n,arg,1,3);
     bsup=arg[0];seq=arg[2];
-    newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&vsup);
+    newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&vsup);
     bseq=addseqright(bseq,geninsertvar(bsup,newleaf(vsup)));
     genblock(bseq,-1);
     newdecl(0,gp->spec==GPvectorsmall?Gvecsmall:Gvec,-1,&ret);
@@ -560,8 +565,8 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
       Gtype tv1,tv2;
       nb=genlistargs(n,arg,2,5);
       bsup2=arg[0];bsup1=arg[1];
-      newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&vsup1);
-      newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&vsup2);
+      newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&vsup1);
+      newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&vsup2);
       bseq=addseqright(bseq,geninsertvar(bsup1,newleaf(vsup1)));
       bseq=addseqright(bseq,geninsertvar(bsup2,newleaf(vsup2)));
       genblock(bseq,-1);
@@ -626,7 +631,7 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
     bl1=newblock();
     if (tree[arg[0]].f!=Fsmall /*&& tree[arg[0]].f!=Fentry*/)
     {
-      newdecl((1<<Cauto)|(1<<Creturn),Gvoid,-1,&var);
+      newdecl((1<<Cauto)|(1<<Creturn),Gempty,-1,&var);
       aseq=geninsertvar(arg[0],var);
       arg[0]=newleaf(var);
     }
@@ -730,18 +735,25 @@ int gentypefuncspec(int n, gpfunc *gp)
       tree[n].m=(tree[arg[0]].m|tree[arg[1]].m)&MODHERIT;
     return Gvoid;
   case GPreturn:
-    tree[n].m=(1<<Mterm);
-    if (y>=0)
     {
-      long t=lfunc[currfunc].type;
-      gentype(y);
-      tree[n].m|=tree[y].m&MODHERIT;
-      t=(t==Gvoid?tree[y].t:typemax[t][tree[y].t]);
-      lfunc[currfunc].type=t;
+      gpfunc *gp=lfunc+currfunc;
+      long t;
+      if (y>=0)
+      {
+        gentype(y);
+        tree[n].m=(1<<Mterm)|(tree[y].m&MODHERIT);
+        t=typemax[gp->type][tree[y].t];
+      }
+      else
+      {
+        tree[n].m=(1<<Mterm);
+        t=typemax[gp->type][Gvoid];
+      }
+      if(debug==2)
+        fprintf(stderr,"%s returned %s now %s (%s)\n",gp->gpname,Gname[gp->type],Gname[t],Gname[tree[y].t]);
+      gp->type=t;
     }
-    else
-      lfunc[currfunc].type=Gvoid;
-    return Gvoid;
+    return Gempty;
   case GPinstall:
   case GPlocal:
   case GPglobal:
@@ -759,65 +771,61 @@ int gentypefuncspec(int n, gpfunc *gp)
       die(n,"Internal error : gentypefuncspec called with no spec");
   }
 }
-void genpercent(FILE *fout, int n)
+
+void genformat(FILE *fout, int cnb, int *carg)
 {
-  char *s=value[tree[n].x].val.str;
-  while(*s)
+  int arg[STACKSZ];
+  int i,j;
+  int larg[1];
+  gpdescarg *rule;
+  gpfunc *gp;
+  int ngp;
+  ngp=findfunction("_formatcode");
+  if (ngp<0) die(-1,"Cannot find description of _formatcode");
+  gp=lfunc+ngp;
+  for(j=0;j<cnb;j++)
   {
-    if (*s=='%')
-      fputc('%',fout);
-    fputc(*s++,fout);
-  }
-}
-void genformat(FILE *fout, int nb, int *arg)
-{
-  int i;
-  for(i=0;i<nb;i++)
-  {
-    switch(ctype[tree[arg[i]].t])
+    int nb=listcalltostack(carg[j],OPcat,arg,STACKSZ,"concatenation",carg[j]);
+    for(i=0;i<nb;i++)
     {
-      case Vbptr:
-      case Vsmall:
-	if (tree[arg[i]].f!=Fsmall)
-	  fprintf(fout,"%%ld");
-        else if (tree[arg[i]].t==Gvar)
-          fprintf(fout,"%%Z");
-	else
-          fprintf(fout,"%ld",tree[arg[i]].x);
-	break;
-      case Vgen:
-	fprintf(fout,"%%Z");
-	break;
-      case Vstr:
-	if (tree[arg[i]].f!=Fstring)
-	  fprintf(fout,"%%s");
-	else
-	  genpercent(fout,arg[i]);
-	break;
-      case Vvoid:
-	break;
-      case Vulong:
-	fprintf(fout,"%%lu");
-	break;
-      case Vgptr:
-	die(-1,"Printing gptr doesn't make sense");
+      int n=arg[i];
+      if (n==GNOARG) continue;
+      larg[0]=n;
+      rule=descfindrules(1, larg, gp);
+      if (!rule) die(n,"No format for %s arg",Gname[tree[n].t]);
+      gencodedesc(fout,1,larg,rule,n);
     }
   }
 }
-void genformatarg(FILE *fout, int nb, int *arg)
+
+void genformatarg(FILE *fout, int cnb, int *carg)
 {
-  int i;
-  for(i=0;i<nb;i++)
-    if (tree[arg[i]].f!=Fstring && tree[arg[i]].f!=Fsmall)
+  int arg[STACKSZ];
+  int i,j;
+  int larg[1];
+  gpdescarg *rule;
+  gpfunc *gp;
+  int ngp;
+  ngp=findfunction("_formatcode");
+  if (ngp<0) die(-1,"Cannot find description of _formatcode");
+  gp=lfunc+ngp;
+  for(j=0;j<cnb;j++)
+  {
+    int nb=listcalltostack(carg[j],OPcat,arg,STACKSZ,"concatenation",carg[j]);
+    for(i=0;i<nb;i++)
     {
-      fprintf(fout,",");
-      if (ctype[tree[arg[i]].t]==Vbptr)
-        fprintf(fout,"*");
-      if (tree[arg[i]].t==Gvar)
-        gencast(fout,arg[i],Gpol);
-      else
-        gencode(fout,arg[i]);
+      int n=arg[i];
+      if (n==GNOARG) continue;
+      larg[0]=n;
+      rule=descfindrules(1, larg, gp);
+      if (!rule) die(n,"Not format for %s arg",Gname[tree[n].t]);
+      if (rule->type!=Gvoid)
+      {
+        fprintf(fout,",");
+        gencast(fout,arg[i],rule->type);
+      }
     }
+  }
 } 
 
 void geninitfunc(FILE *fout)
@@ -840,9 +848,9 @@ void geninitfunc(FILE *fout)
 #endif
             genindent(fout);
             if (ctype[ctxstack[i].t]==Vgen)
-              fprintf(fout,"%s = polx[fetch_user_var(\"%s\")];\n",v->var,v->var);
+              fprintf(fout,"%s = polx[fetch_user_var(\"%s\")];\n",v->cvar,v->var);
             else if (ctxstack[i].t==Gvar)
-              fprintf(fout,"%s = fetch_user_var(\"%s\");\n",v->var,v->var);
+              fprintf(fout,"%s = fetch_user_var(\"%s\");\n",v->cvar,v->var);
 #if TEST_UNITIALISED
           }
           break;
@@ -864,6 +872,7 @@ void gencheckargs(FILE *fout)
   int nb;
   context *fc=block+uf->bl;
   gpfunc *gp;
+  gpdescarg *rule;
   int ngp;
   ngp=findfunction("_badtype");
   if (ngp<0) die(nfunc,"Cannot find description of _badtype");
@@ -875,30 +884,28 @@ void gencheckargs(FILE *fout)
   {
     ctxvar *v;
     int doelse=0;
+    gpdescarg *rule;
     larg[0]=detag(arg[i]);
     if (tree[larg[0]].f==Faffect)
       larg[0]=tree[larg[0]].x;
     larg[0]=detag(larg[0]);
     v=getvarinblock(larg[0], fc);
-    if (v->t!=Gnotype && ctype[v->t]==Vgen && v->initval>0)
+    if (ctype[v->t]==Vgen && v->initval>0)
     {
       doelse=1;
       genindent(fout);
-      fprintf(fout,"if ( !%s ) %s = ",v->var,v->var);
+      fprintf(fout,"if ( !%s ) %s = ",v->cvar,v->cvar);
       gencast(fout,v->initval,v->t);
       fprintf(fout,";\n");
     }
-    if (v->t!=Gnotype)
+    tree[larg[0]].t=v->t;
+    tree[larg[0]].comment=-1;
+    rule=descfindrules(1, larg, gp);
+    if (!rule || rule->args[0].v != v->t) continue;
+    genindent(fout);
+    if(doelse) fprintf(fout,"else ");
+    switch(rule->type)
     {
-      gpdescarg *rule;
-      tree[larg[0]].t=v->t;
-      tree[larg[0]].comment=-1;
-      rule=descfindrules(1, larg, gp);
-      if (!rule || rule->args[0].v != v->t) continue;
-      genindent(fout);
-      if(doelse) fprintf(fout,"else ");
-      switch(rule->type)
-      {
       case Gbool:
         fprintf(fout,"if (");
         gencodedesc(fout,1,larg,rule,nfunc);
@@ -916,9 +923,8 @@ void gencheckargs(FILE *fout)
       default:
         gencodedesc(fout,1,larg,rule,nfunc);
         break;
-      } 
-      fprintf(fout,";\n");
-    }
+    } 
+    fprintf(fout,";\n");
   }
 }
 
@@ -1250,7 +1256,7 @@ void genentryspec(FILE *fout, int n, gpfunc *gp)
     break;
   case GPprint:
     nb=genlistargs(n,arg,0,STACKSZ);
-    fprintf(fout,"fprintferr(\"");
+    fprintf(fout,"pariputsf(\"");
     genformat(fout,nb,arg);
     if (!isfunc(n,"print1") && !isfunc(n,"printp1"))
       fprintf(fout,"\\n");
@@ -1308,7 +1314,7 @@ void genentryspec(FILE *fout, int n, gpfunc *gp)
     for(i=0;i<bl->g.n;i++)
     {
       if (i) fprintf(fout,", ");
-      fprintf(fout,"&%s",ctxstack[bl->gcvar[i]].var);
+      fprintf(fout,"&%s",ctxstack[bl->gcvar[i]].cvar);
     }
     fprintf(fout," }");
     break;
@@ -1326,7 +1332,7 @@ void genentryspec(FILE *fout, int n, gpfunc *gp)
     }
     else if (bl->gc&(1<<GCupto))
     {
-      char *varname=ctxstack[bl->gcvar[0]].var;
+      const char *varname=ctxstack[bl->gcvar[0]].cvar;
       fprintf(fout,"%s = gerepileupto(",varname);
       gencode(fout,arg[0]);
       fprintf(fout,", %s);\n",varname);
