@@ -24,11 +24,11 @@ extern int initnode;
 int newdecl(int flag, Gtype t, int initval, int *v)
 {
   char s[33];
-  sprintf(s,"p%d",newprivvar++);
-  *v=newnode(Fentry,newentry(s),-1);
+  sprintf(s,"%d",newprivvar++);
+  *v=newnode(Fentry,newentry(strdup(s)),-1);
   return pushvar(*v,flag,t,initval);
 }
-int newcall(char *s, int y)
+int newcall(const char *s, int y)
 {
   return newnode(Fentryfunc,newentry(s),y);
 }
@@ -131,7 +131,11 @@ void genblockdeclaration(int args, int n, int flag)
     switch(tree[stack[i]].f)
     {
     case Ftag:
-      pushvar(tree[stack[i]].x,flag,tree[stack[i]].y,-1);
+      var=tree[stack[i]].x;
+      tv=tree[stack[i]].y;
+      if (tree[var].f!=Fentry)
+        die(stack[i],"Incorrect declaration");
+      pushvar(var,flag,tv,-1);
       break;
     case Fentry:
       pushvar(stack[i],flag,mint,-1);
@@ -172,7 +176,7 @@ void genblockdeffunc(int n)
 {
   int funcid=tree[n].x;
   int seq=tree[n].y;
-  char *name=entryname(funcid);
+  const char *name=entryname(funcid);
   int args=tree[funcid].y;
   int bl1,bl2,gpm,ret,gltop;
   int savcf,savnpv;
@@ -203,7 +207,7 @@ void genblockdeffunc(int n)
     pushvar(gltop,1<<Cuser,Gulong,newcall("_avma",-1));
   }
   /*declare return var*/
-  vret=newdecl((1<<Cauto)|(1<<Creturn),Gvoid,-1,&ret);
+  vret=newdecl((1<<Cauto)|(1<<Creturn),Gempty,-1,&ret);
   
   /*create external block*/
   if (autogc)
@@ -275,7 +279,7 @@ void genblockmatrixl(int n, int x, int y)
   int savx,bl1,bl2;
   int aseq,bseq=-1;
   /*we have x[y,] x is a matrix, y a small*/
-  newdecl((1<<Cauto)|(1<<Cconst),Gvoid,-1,&vx);
+  newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&vx);
   newdecl((1<<Cconst),Gsmall,-1,&vy);
   newdecl((1<<Cconst),Glg,-1,&vsup);
   bsup=newcall("length",newnode(Ftag,newleaf(vx),Gvec));
@@ -301,6 +305,37 @@ void genblockmatrixl(int n, int x, int y)
   aseq=newcall("vector",listtoseq(arg,6));
   makeblocks(bl1,bl2,n,bseq,aseq,ret,savx);
 }
+void genblockaffmatrixl(int n, int x, int y, int z)
+{
+  int arg[6];
+  int var,bsup,seq,ret;
+  int vy,vz,vsup;
+  int savx,bl1,bl2;
+  int aseq,bseq=-1;
+  /*we have x[y,] x is a matrix, y a small, z is a vector*/
+  newdecl((1<<Cconst),Gsmall,-1,&vy);
+  newdecl((1<<Cconst),Gvec,-1,&vz);
+  newdecl((1<<Cconst),Glg,-1,&vsup);
+  bsup=newcall("length",newnode(Ftag,newleaf(x),Gvec));
+  bseq=addseqright(bseq,geninsertvar(z,vz));
+  bseq=addseqright(bseq,geninsertvar(y,vy));
+  bseq=addseqright(bseq,geninsertvar(bsup,vsup));
+  genblock(bseq,-1);
+  savx=s_ctx.n;
+  bl1=newblock();
+  bl2=newblock();
+  newdecl(0,Gsmall,-1,&var);
+  seq=newnode(Ffacteurmat,newleaf(x),
+      newnode(Fmatrix,newleaf(vy),newleaf(var)));
+  arg[0]=newnode(Faffect,var,newnode(Fsmall,1,-1));
+  arg[1]=newopcall(OPle,newleaf(var),vsup);
+  arg[2]=newnode(Ffacteurmat,newleaf(vz),newnode(Fmatrix,newleaf(var),-1));
+  arg[2]=geninsertvar(arg[2],seq);
+  genblock(arg[2],-1);
+  arg[3]=newopcall(OPpp,newleaf(var),-1);
+  aseq=newcall("for",listtoseq(arg,4));
+  makeblocks(bl1,bl2,n,bseq,aseq,vz,savx);
+}
 void genblockvector(int n)
 {
   int arg[STACKSZ];
@@ -401,6 +436,21 @@ void genblock(int n, int p)
       /*else it is a vector*/
       genblockvector(n);
     break;
+  case Faffect:
+    x=detag(x);
+    if (tree[x].f==Ffacteurmat && tree[tree[x].y].f==FmatrixL)
+    {
+      int xx=tree[x].x;
+      int xy=tree[tree[x].y].x;
+      genblockaffmatrixl(n,xx,xy,y); 
+    }
+    else
+    {
+      genblock(x, n);
+      genblock(y, n);
+      /*  x=tree[n].x; y=tree[n].y;*/
+    }
+    break;
   case Fmatrix:
   case FmatrixR:
     genblock(x,n);
@@ -422,7 +472,7 @@ void genblock(int n, int p)
     {
       genblock(x,n);
       genblock(y,n);
-    /*  x=tree[n].x; y=tree[n].y;*/
+      /*  x=tree[n].x; y=tree[n].y;*/
     }
     break;	
   case Frefarg:
@@ -430,9 +480,14 @@ void genblock(int n, int p)
     if (findfunction(entryname(n))==-1) /*it is not a function call*/
     {
       if (getvar(n)==-1)
-      {/*The variable has not been declared*/
-	pushvar(n,(1<<Cuser)|(1<<Cglobal),Gnotype,-1);
-	if(warn) warning(n,"variable undeclared");
+      {/*The variable has not been declared.
+        * We declare it as a function-local variable initialized
+        * to 'var. The global flag is set to avoid being block-local.
+        * It is discarded in genblockdeffunc. This can be changed.
+        */
+        int iv=newnodecom(Fconst,newquotevalue(entryname(n)),-1,-1);
+        pushvar(n,(1<<Cuser)|(1<<Cglobal),Ggen,iv);
+        if(warn) warning(n,"variable undeclared");
       }
       break;
     }/*else it is a function call*/
@@ -454,9 +509,9 @@ void genblock(int n, int p)
     {
       int nf=findfunction(value[x].val.str);
       if (lfunc[nf].spec>0)
-	genblockfuncspec(n,p,lfunc+nf);
+        genblockfuncspec(n,p,lfunc+nf);
       else
-	genblock(y,n);
+        genblock(y,n);
     }
     break;
   case Fdeffunc:
@@ -465,9 +520,7 @@ void genblock(int n, int p)
     genblockdeffunc(n);
     break;
   case Fconst:
-  case Fstring:
   case Fsmall:
-  case FtrucQ:
   case Fgnil:
     break;
   case Ftag:
