@@ -25,9 +25,9 @@ inittrans(int n,int p)
   int i;
   for(i=0;i<Gnbtype;i++)
   {
-    if (typemax[i][n]==n)
+    if (is_subtype(i,n))
       typemax[p][i]=typemax[i][p]=p;
-    if (typemax[p][i]==i)
+    if (is_subtype(p,i))
       typemax[i][n]=typemax[n][i]=i;
   }
 }
@@ -36,8 +36,8 @@ initmax(int n, int p)
 {
   int i,m=Gnotype;
   for(i=0;i<Gnbtype;i++)
-    if (typemax[n][i]==i && typemax[p][i]==i)
-      if(m==Gnotype || typemax[i][m]==m)
+    if (is_subtype(n,i) && is_subtype(p,i))
+      if(m==Gnotype || is_subtype(i,m))
 	m=i;
   return m;
 }
@@ -76,9 +76,13 @@ void inittype(void)
     {
       int ti=typedesc[i];
       int tn=typedesc[n];
-      inittrans(ti,tn);
+      typemax[ti][tn]=typemax[tn][ti]=tn;
     }
   }
+  for(i=0;i<Gnbtype;i++)
+    for(j=0;j<Gnbtype;j++)
+      if (typemax[i][j]==j)
+        inittrans(i,j);
   for(i=0;i<Gnbtype;i++)
     for(j=0;j<Gnbtype;j++)
       if (typemax[i][j]==Gnotype)
@@ -110,7 +114,7 @@ void gentypedeffunc(int n)
 {
   int funcid=tree[n].x;
   int seq=tree[n].y;
-  char *name=entryname(funcid);
+  const char *name=entryname(funcid);
   int savcf;
   ctxvar *ret;
   int tf;
@@ -129,14 +133,14 @@ void gentypedeffunc(int n)
   /*get return var*/
   ret=getvarinblock(fc->ret,block+tree[seq].x);
   /*update prototype*/
-  if ((tf==Gvoid && ret->t!=Gvoid) || typemax[tf][ret->t]!=tf)
+  if (!is_subtype(ret->t,tf))
   {
     if(debug==2)
       fprintf(stderr,"%s returned %s now %s:",gp->gpname,Gname[tf],Gname[ret->t]);
-    tf=gp->type=(tf==Gvoid?ret->t:typemax[tf][ret->t]);
+    tf=gp->type=typemax[tf][ret->t];
     if (tf==Gnegbool) tf=gp->type=Gbool;
     if (tf==Glg) tf=gp->type=Gsmall;
-    if (tf==Gstr) tf=gp->type=Ggen;
+    if (tf==Gstr) tf=gp->type=Ggenstr;
     if(debug==2)
       fprintf(stderr,"casted to %s\n",Gname[tf]);
      lastpass++;
@@ -229,7 +233,7 @@ Gtype gentype(int n)
     tree[n].m=0;
     break;
   case Ffacteurmat:
-    tree[n].t=(tx==Gvecsmall)?Gsmall:Ggen;
+    tree[n].t=is_subtype(tx,Gvecsmall)?Gsmall:Ggen;
     tree[n].m=(mx&MODHERIT)|(1<<Mcopy);
     if (tree[y].f!=FmatrixL && tree[n].t==Ggen)
       tree[n].m|=1<<Mlong;
@@ -240,21 +244,21 @@ Gtype gentype(int n)
       tree[n].t=tx;/*Probably it should be ty*/
       tree[n].m=(mx|my)&MODHERIT;
       z=detag(tree[n].x);
-      if (ty!=Gnotype && typemax[tx][ty]!=tx && tree[z].f==Fentry)
+      if (!is_subtype(ty,tx) && tree[z].f==Fentry)
       {
 	int v=getvar(z);
 	if (v!=-1)
 	{
 	  if (!(ctxstack[v].flag&(1<<Cauto)))
 	  {
-	    if ((ctxstack[v].flag&(1<<Cuser)) && default_type!=Gnotype) 
+	    if ((ctxstack[v].flag&(1<<Cuser))) 
 	      warning(n,"Type clash :%s!>=%s",Gname[tx],Gname[ty]);
 	  }
 	  else
 	  {
-	    ctxstack[v].t=(tx==Gvoid?ty:typemax[tx][ty]);
+	    ctxstack[v].t=typemax[tx][ty];
 	    lastpass++;	
-	    if (debug) fprintf(stderr,"casting %s from %s to %s\n",ctxstack[v].var,Gname[tx],Gname[ty]);
+	    if (debug) fprintf(stderr,"casting %s from %s to %s now %s.\n",ctxstack[v].var,Gname[tx],Gname[ty],Gname[ctxstack[v].t]);
 	    if(debug>=2)
 	    {
 	      printnode(stderr,n);
@@ -272,11 +276,11 @@ Gtype gentype(int n)
     }
     break;
   case Fconst:
+    tree[n].m=0;
     switch(value[x].type)
     {
     case CSTsmall:      
       tree[n].t=Gsmall;
-      tree[n].m=0;
       break;
     case CSTsmallreal:
       tree[n].t=Greal;
@@ -284,22 +288,19 @@ Gtype gentype(int n)
       break;
     case CSTexpr:
       tree[n].t=isanint(value[x].val.str)?Gint:Greal;
-      tree[n].m=0;
+      break;
+    case CSTstr:
+      tree[n].t=Gstr;
+      break;
+    case CSTquote:
+      tree[n].t=Gvar;
       break;
     }
     break;
-  case Fstring:
-    tree[n].t=Gstr;
-    tree[n].m=0;
-    break;
   case Fsmall:
     tree[n].t=Gsmall;
     tree[n].m=0;
     break;
-  case FtrucQ:
-    tree[n].t=Gvar;
-    tree[n].m=0;
-    break;
   case Femptyvec:
     tree[n].t=Gvec;
     tree[n].m=0;
@@ -326,18 +327,10 @@ Gtype gentype(int n)
     c=getvar(n);
     if (c>=0)
     {	  
-      if (ctxstack[c].t!=Gnotype)
-      {
-        tree[n].t=ctxstack[c].t;
-        tree[n].m=(1<<Mvar);
-        if (ctxstack[c].flag&(1<<Carg))
-          tree[n].m|=(1<<Mcopy);
-      }
-      else
-      {
-        tree[n].t=Ggen;
-        tree[n].m=0;
-      }
+      tree[n].t=ctxstack[c].t;
+      tree[n].m=(1<<Mvar);
+      if (ctxstack[c].flag&(1<<Carg))
+        tree[n].m|=(1<<Mcopy);
     }
     else
       die(n,"Internal error: extra variable `%s' in gentype",value[x].val.str);
@@ -345,26 +338,30 @@ Gtype gentype(int n)
   case Fentryfunc:
     nf=findfunction(entryname(n));
     gp=lfunc+nf;
-    if (nf>=0 && gp->type==Gnotype && gp->spec>0)
-      tree[n].t=gentypefuncspec(n,gp);
-    else if (nf>=0 && gp->dsc && (t=gentypefuncdesc(n,gp))!=Gnotype)
-      tree[n].t=t;
-    else
+    if (nf>=0) 
     {
-      ty=(y==-1)?Gnotype:gentype(y);
-      my=(y==-1)?0:tree[y].m;
-      if (nf>=0)
+      if (gp->type==Gnotype && gp->spec>0)
+        /* is a special function ? */
+        tree[n].t=gentypefuncspec(n,gp);
+      else if (gp->dsc && (t=gentypefuncdesc(n,gp))!=Gnotype) 
+        /* else has the function a description ?*/ 
+        tree[n].t=t;
+      else
+        /* else use the function a type */ 
       {
+        gentype(y);
         tree[n].t=gp->type;
+        my=(y==-1)?0:tree[y].m;
         tree[n].m=(gp->mode&(~(1<<Msemicomma)))|(my&MODHERIT);
       }
-      else
-      {
-        tree[n].t=default_type;
-        tree[n].m=my&MODHERIT;
-        if (default_type!=Gnotype)
-          warning(n,"function prototype is unknown");
-      }
+    }
+    else
+    {
+      gentype(y);
+      my=(y==-1)?0:tree[y].m;
+      tree[n].t=Ggen;
+      tree[n].m=my&MODHERIT;
+      warning(n,"function prototype is unknown");
     }
     break;
   case Fdeffunc:
